@@ -36,6 +36,26 @@
 
 #include "autoconfig.h"
 
+#ifdef _WIN32
+
+// This needs to be defined before we include any Windows include and config.h is usually at the top
+// of files (or we can put it there), so it's the right place. We need this to get, e.g. inet_pton
+// from ws2tcpip.h
+#if _WIN32_WINNT < 0x0600
+#undef _WIN32_WINNT
+#define _WIN32_WINNT 0x0600
+#endif
+
+#ifdef _MSC_VER
+/* no ssize_t defined for VC */
+#include <cstdint>
+typedef intptr_t ssize_t;
+#define strncasecmp _strnicmp
+#define strcasecmp _stricmp
+#define getpid _getpid
+#endif /* _MSC_VER */
+
+#endif // _WIN32
 
 /*!
  *  \name Compile time configuration options
@@ -175,14 +195,15 @@
 /*!
  * \name DEFAULT_SOAP_CONTENT_LENGTH
  *
+ * Misnomer because also used for GENA POST data.
  * SOAP messages will read at most {\tt DEFAULT_SOAP_CONTENT_LENGTH} bytes.  
  * This prevents devices that have a misbehaving web server to send 
  * a large amount of data to the control point causing it to crash.  
  * This can be adjusted dynamically with {\tt UpnpSetMaxContentLength}.
- *
+ * This is used both for POST and response data and used to be 16k. Now adjusted to 2MB.
  * @{
  */
-#define DEFAULT_SOAP_CONTENT_LENGTH 16000
+#define DEFAULT_SOAP_CONTENT_LENGTH 2000*1024
 /* @} */
 
 
@@ -195,7 +216,7 @@
  *
  * @{
  */
-#define NUM_SSDP_COPY  2
+#define NUM_SSDP_COPY  3
 /* @} */
 
 
@@ -208,19 +229,9 @@
  *
  * @{
  */
-#define SSDP_PAUSE  100u
+#define SSDP_PAUSE  100
 /* @} */
 
-/*!
- * \name WEB_SERVER_BUF_SIZE
- * 
- * This configuration parameter sets the maximum buffer size for the 
- * webserver. The default value is 1MB.
- *
- * @{
- */
-#define WEB_SERVER_BUF_SIZE  (size_t)(1024*1024)
-/* @} */
 
 /*!
  * \name WEB_SERVER_CONTENT_LANGUAGE
@@ -267,34 +278,6 @@
 /* @} */
 
 
-/*!
- * \name MAX_SEARCH_TIME
- *
- * The {\tt MAX_SEARCH_TIME} is the maximum time
- * allowed for an SSDP search by a control point. Searching for greater than
- * this time automatically results in a search for this amount.  The default 
- * value is 80 seconds.
- *
- * @{
- */
-#define MAX_SEARCH_TIME 80
-/* @} */
-
-
-/*!
- * \name MIN_SEARCH_TIME
- *
- * The {\tt MIN_SEARCH_TIME} is the minimumm time
- * allowed for an SSDP search by a control point. Searching for less than
- * this time automatically results in a search for this amount.  The default 
- * value is 2 seconds.
- *
- * @{
- */
-#define MIN_SEARCH_TIME 2
-/* @} */
-
-
 /*!
  * \name AUTO_ADVERTISEMENT_TIME
  *
@@ -368,54 +351,7 @@
 /* @} */
 
 
-/*!
- * \name Module Exclusion
- *
- * Depending on the requirements, the user can selectively discard any of 
- * the major modules like SOAP, GENA, SSDP or the Internal web server. By 
- * default everything is included inside the SDK.  By setting any of
- * the values below to 0, that component will not be included in the final
- * SDK.
- * \begin{itemize}
- *   \item {\tt EXCLUDE_SOAP[0,1]}
- *   \item {\tt EXCLUDE_GENA[0,1]}
- *   \item {\tt EXCLUDE_SSDP[0,1]}
- *   \item {\tt EXCLUDE_DOM [0,1]}
- *   \item {\tt EXCLUDE_MINISERVER[0,1]}
- *   \item {\tt EXCLUDE_WEB_SERVER[0,1]}
- *   \item {\tt EXCLUDE_JNI[0,1]}
- * \end{itemize}
- *
- * @{
- */
-#define EXCLUDE_SSDP 0
-#define EXCLUDE_SOAP 0
-#define EXCLUDE_GENA 0
-#define EXCLUDE_DOM  0
-#define EXCLUDE_MINISERVER 0
-#define EXCLUDE_WEB_SERVER 0
-#ifdef USE_JNI
-#    define EXCLUDE_JNI 0
-#else
-#    define EXCLUDE_JNI 1
-#endif
-/* @} */
-
-    
-/*!
- * \name DEBUG_TARGET
- *
- * The user has the option to redirect the library output debug messages 
- * to either the screen or to a log file.  All the output messages with 
- * debug level 0 will go to {\tt upnp.err} and messages with debug level 
- * greater than zero will be redirected to {\tt upnp.out}.
- *
- * @{
- */
-#define DEBUG_TARGET        1   
-/* @} */
-
-
+   
 /*!
  * \name Other debugging features
  *
@@ -431,22 +367,8 @@
 #define DEBUG_MSERV        0
 #define DEBUG_DOM        0
 #define DEBUG_HTTP        0
-#define DEBUG_API        0
 
 
-/*
- * @} Compile time configuration options
- */
-
-
-/***************************************************************************
- * Do not change, Internal purpose only!!!
- ***************************************************************************/ 
-
-/*!
- * @{
- */
-
 
 /*
  * Set additional defines based on requested configuration 
@@ -471,7 +393,6 @@
 #endif
 
 /* configure --enable-ssdp */
-#undef EXCLUDE_SSDP
 #if UPNP_HAVE_SSDP
 #       define EXCLUDE_SSDP 0
 #else
@@ -479,7 +400,6 @@
 #endif
 
 /* configure --enable-soap */
-#undef EXCLUDE_SOAP
 #if UPNP_HAVE_SOAP
 #    define EXCLUDE_SOAP 0
 #else
@@ -487,15 +407,12 @@
 #endif
 
 /* configure --enable-gena */
-#undef EXCLUDE_GENA
 #if UPNP_HAVE_GENA
 #       define EXCLUDE_GENA 0
 #else
 #       define EXCLUDE_GENA 1
 #endif
 
-#undef  EXCLUDE_WEB_SERVER
-#undef  EXCLUDE_MINISERVER
 #ifdef  INTERNAL_WEB_SERVER
 #    define EXCLUDE_WEB_SERVER 0
 #    define EXCLUDE_MINISERVER 0
