@@ -33,35 +33,30 @@
 /* Misc HTTP-related utilities */
 
 #include "config.h"
-
 #include "httputils.h"
 
-#include <cctype>
 #include <cinttypes>
-#include <climits>
-#include <cstdio>
+#include <cstdint>
 #include <cstdlib>
 #include <cstring>
-#include <cstdint>
-
-#include <string>
-#include <sstream>
+#include <ctime>
 #include <iostream>
-#include <map>
+#include <unordered_map>
+#include <sstream>
+#include <string>
+#include <mutex>
 
 #include <curl/curl.h>
+#include <microhttpd.h>
 
-#include "UpnpInet.h"
 #include "genut.h"
 #include "statcodes.h"
 #include "upnp.h"
 #include "upnpdebug.h"
 #include "uri.h"
-#include "genut.h"
-#include "upnpapi.h"
 
-static const std::string bogus_soap_post{"SMPOST"};
-static const std::map<std::string, int> Http_Method_Table {
+static constexpr std::string_view bogus_soap_post{"SMPOST"};
+static const std::unordered_map<std::string_view, int> Http_Method_Table {
     {"GET", HTTPMETHOD_GET},
     {"HEAD", HTTPMETHOD_HEAD},
     {"M-POST", HTTPMETHOD_MPOST},
@@ -73,7 +68,7 @@ static const std::map<std::string, int> Http_Method_Table {
     {bogus_soap_post, SOAPMETHOD_POST},
 };
 
-static const std::map<std::string, int> Http_Header_Names {
+static const std::unordered_map<std::string_view, int> Http_Header_Names {
     {"accept", HDR_ACCEPT},
     {"accept-charset", HDR_ACCEPT_CHARSET},
     {"accept-encoding", HDR_ACCEPT_ENCODING},
@@ -113,19 +108,28 @@ void MHDTransaction::copyClientAddress(struct sockaddr_storage *dest) const
 {
     if (nullptr == dest)
         return;
-    if (nullptr == client_address) {
-        *dest = {};
-        return;
-    }
-    if (client_address->ss_family == AF_INET) {
-        memcpy(dest, client_address, sizeof(struct sockaddr_in));
+    if (client_address.ss_family == AF_INET) {
+        memcpy(dest, &client_address, sizeof(struct sockaddr_in));
+    } else if (client_address.ss_family == AF_INET6) {
+        memcpy(dest, &client_address, sizeof(struct sockaddr_in6));
     } else {
-        memcpy(dest, client_address, sizeof(struct sockaddr_in6));
-    }        
+        *dest = {};
+    }
 }
 
-bool MHDTransaction::copyHeader(const std::string& name,
-                                std::string& value)
+void MHDTransaction::copyToClientAddress(const struct sockaddr *src)
+{
+    memset(&client_address, 0, sizeof(client_address));
+    if (nullptr == src)
+        return;
+    if (src->sa_family == AF_INET) {
+        memcpy(&client_address, src, sizeof(struct sockaddr_in));
+    } else if (src->sa_family == AF_INET6) {
+        memcpy(&client_address, src, sizeof(struct sockaddr_in6));
+    }
+}
+
+bool MHDTransaction::copyHeader(const std::string& name, std::string& value)
 {
     auto it = headers.find(stringtolower(name));
     if (it == headers.end()) {
@@ -194,8 +198,7 @@ int http_FixStrUrl(const std::string& surl, uri_type *fixed_url)
  *    UPNP_E_SUCCESS
  *    UPNP_E_INVALID_URL
  ************************************************************************/
-int http_Download(const char *_surl, int timeout_secs,
-                  char **document, size_t *, char *content_type)
+int http_Download(const char *_surl, int timeout_secs, char **document, size_t *, char *content_type)
 {
     uri_type url;
     UpnpPrintf(UPNP_INFO, HTTP, __FILE__, __LINE__, "http_Download: %s\n",_surl);
@@ -234,8 +237,7 @@ int http_Download(const char *_surl, int timeout_secs,
     }
     long http_status;
     curl_easy_getinfo (easy, CURLINFO_RESPONSE_CODE, &http_status);
-    UpnpPrintf(UPNP_INFO, HTTP, __FILE__, __LINE__, "Response. Status %ld\n",
-               http_status);
+    UpnpPrintf(UPNP_INFO, HTTP, __FILE__, __LINE__, "Response. Status %ld\n", http_status);
 
     curl_easy_cleanup(easy);
     curl_slist_free_all(list);
@@ -252,7 +254,7 @@ int http_Download(const char *_surl, int timeout_secs,
 
     auto it = http_headers.find("content-length");
     if (it != http_headers.end()) {
-        uint64_t sizefromheaders = atoll(it->second.c_str());
+        auto sizefromheaders = static_cast<uint64_t>(atoll(it->second.c_str()));
         if (sizefromheaders != data.size()) {
             UpnpPrintf(UPNP_INFO, HTTP, __FILE__, __LINE__,
                        "Response content-length %" PRIu64
@@ -273,7 +275,7 @@ int http_Download(const char *_surl, int timeout_secs,
         return 0;
     }
 
-    return http_status;
+    return static_cast<int>(http_status);
 }
 
 /************************************************************************
@@ -298,7 +300,7 @@ int http_Download(const char *_surl, int timeout_secs,
 int http_SendStatusResponse(MHDTransaction *mhdt, int status_code)
 {
     std::ostringstream body;
-    body <<    "<html><body><h1>" << status_code << " " << 
+    body << "<html><body><h1>" << status_code << " " << 
         http_get_code_text(status_code) << "</h1></body></html>";
     mhdt->response = MHD_create_response_from_buffer(
         body.str().size(), const_cast<char*>(body.str().c_str()), MHD_RESPMEM_MUST_COPY);
@@ -314,8 +316,7 @@ bool has_xml_content_type(MHDTransaction *mhdt)
 
     auto it = mhdt->headers.find("content-type");
     if (it == mhdt->headers.end()) {
-        UpnpPrintf(UPNP_INFO, HTTP, __FILE__, __LINE__,
-                   "has_xml_content: no content type header\n");
+        UpnpPrintf(UPNP_INFO, HTTP, __FILE__, __LINE__, "has_xml_content: no content type header\n");
         return false;
     }
     if (strncasecmp(xmlmtype, it->second.c_str(), mtlen)) {
@@ -331,8 +332,7 @@ bool timeout_header_value(std::map<std::string, std::string>& headers,
 {
     auto ittimo = headers.find("timeout");
     if (ittimo == headers.end()) {
-        UpnpPrintf(UPNP_INFO, HTTP, __FILE__, __LINE__,
-                   "timeout_header_value: no timeout header\n");
+        UpnpPrintf(UPNP_INFO, HTTP, __FILE__, __LINE__, "timeout_header_value: no timeout header\n");
         return false;
     }
     stringtolower(ittimo->second);
@@ -350,7 +350,7 @@ bool timeout_header_value(std::map<std::string, std::string>& headers,
 }
 
 #ifdef _WIN32
-struct tm *http_gmtime_r(const time_t *clock, struct tm *result)
+static tm* http_gmtime_r(const time_t* clock, tm* result)
 {
     if (clock == NULL || *clock < 0 || result == NULL)
         return NULL;
@@ -369,19 +369,20 @@ struct tm *http_gmtime_r(const time_t *clock, struct tm *result)
 
 static const std::string& get_sdk_common_info()
 {
+    static std::mutex lmutex;
+    std::lock_guard<std::mutex> llock(lmutex);
     static std::string sdk_common_info;
     if (sdk_common_info.empty()) {
         std::ostringstream ostr;
 #ifdef UPNP_ENABLE_UNSPECIFIED_SERVER
-        ostr << "Unspecified UPnP/1.0 Unspecified";
+        ostr << "Unspecified UPnP/1.1 Unspecified";
 #else /* UPNP_ENABLE_UNSPECIFIED_SERVER */
 #ifdef _WIN32
             OSVERSIONINFO versioninfo;
         versioninfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
 
         if (GetVersionEx(&versioninfo) != 0)
-            ostr << versioninfo.dwMajorVersion << "." <<
-                versioninfo.dwMinorVersion << "." <<
+            ostr << versioninfo.dwMajorVersion << "." << versioninfo.dwMinorVersion << "." <<
                 versioninfo.dwBuildNumber << " " << versioninfo.dwPlatformId
                  << "/" << versioninfo.szCSDVersion;
 #else
@@ -392,7 +393,7 @@ static const std::string& get_sdk_common_info()
 #endif
 #endif /* UPNP_ENABLE_UNSPECIFIED_SERVER */
 
-        ostr << " UPnP/1.0 ";
+        ostr << " UPnP/1.1 ";
         sdk_common_info = ostr.str();
     }
     return sdk_common_info;
@@ -407,6 +408,8 @@ std::string get_sdk_device_info(const std::string& customvalue)
 
 const std::string& get_sdk_client_info(const std::string& newvalue)
 {
+    static std::mutex lmutex;
+    std::lock_guard<std::mutex> llock(lmutex);
     static std::string sdk_client_info;
     if (sdk_client_info.empty() || !newvalue.empty()) {
         // If this was never set, or the client wants to set its name, compute
@@ -426,7 +429,7 @@ std::string make_date_string(time_t thetime)
 
     time_t curr_time = thetime ? thetime : time(nullptr);
     struct tm date_storage;
-    struct tm *date = http_gmtime_r(&curr_time, &date_storage);
+    const struct tm *date = http_gmtime_r(&curr_time, &date_storage);
     if (date == nullptr)
         return {};
     char tempbuf[200];
@@ -460,10 +463,10 @@ std::string query_encode(const std::string& qs)
     return out;
 }
 
-size_t header_callback_curl(char *buffer, size_t size, size_t nitems, void *s)
+size_t header_callback_curl(char *buffer, size_t size, size_t nitems,
+                            std::map<std::string, std::string> *headers)
 {
     size_t bufsize = size * nitems;
-    auto headers = static_cast<std::map<std::string, std::string>*>(s);
     const char *colon = std::strchr(buffer, ':');
     if (nullptr != colon) {
         size_t colpos = colon - buffer;
@@ -481,7 +484,7 @@ size_t header_callback_curl(char *buffer, size_t size, size_t nitems, void *s)
     return bufsize;
 }
 
-size_t write_callback_null_curl(char *buffer, size_t size, size_t nitems, void *)
+size_t write_callback_null_curl(char *buffer, size_t size, size_t nitems, std::string *)
 {
     (void)buffer;
 #if 0
@@ -493,8 +496,23 @@ size_t write_callback_null_curl(char *buffer, size_t size, size_t nitems, void *
     return size*nitems;
 }
 
-size_t write_callback_str_curl(char *buf, size_t sz, size_t nits, void *s)
+// Not worth including upnpapi.h just for this.
+extern size_t g_maxContentLength;
+
+// Needed with older Curl releases (e.g. Debian bullseye). Actually, according to curl.h comments,
+// any value differing from the input size will halt the transfer.
+#ifndef CURL_WRITEFUNC_ERROR
+#define CURL_WRITEFUNC_ERROR 0xFFFFFFFF
+#endif
+
+size_t write_callback_str_curl(char *buf, size_t sz, size_t nits, std::string *s)
 {
-    (static_cast<std::string*>(s))->append(buf, sz * nits);
+    if (s->size() + sz * nits > g_maxContentLength) {
+        std::cerr << "Total read size exceeds maxContentLength\n";
+        UpnpPrintf(UPNP_ERROR, SOAP, __FILE__, __LINE__,
+                   "Total read size exceeds maxContentLength\n");
+        return CURL_WRITEFUNC_ERROR;
+    }
+    s->append(buf, sz * nits);
     return sz * nits;
 }
